
####  Covariance matrix estimation using Approximate Factor Model #######
####  Example 7.8 and Figure 7.7, the Fama-French 100 portfolos ##########


 rm(list=ls())
 
 ############## Daily returns of Fama-French factors and 100 portfolios
 FF = matrix(scan("F-F_Research_Data_Factors_daily.txt", skip=5),  ncol=5, byrow=T)	
 port100 = matrix(scan("100_Portfolios_10x10_daily.txt", skip=18, n=101*252), ncol=101, byrow=T)	

 port100[,2:101] = port100[,2:101] - FF[,5]

 #################### Least Squares fitting ####################
 N = 63            #Length of data
 X = FF[(252-N+1):252,2:4]
 Y = data.frame(port100[(252-N+1):252,2:101])    #Market factor and portfolios

 resid = resid(lsfit(X, Y))
 B    = lsfit(X, Y)$coef
 B    = t(B[-1,])        #take out intercept
 SigmaE = t(resid) %*% resid / (N-4)

 ####################### Thresholding #########################
 SDs = diag(SigmaE)^{1/2}
 SigmaEcorr = diag(1/SDs) %*% SigmaE %*%  diag(1/SDs)
 
 SigmaEcorr[abs(SigmaEcorr) < 0.1] = 0               #thresholding
 zeros = sum(abs(SigmaEcorr) < 0.1)/2                ### Number of zeros
 print(zeros)
 
 lambda = min(eigen(SigmaEcorr)$values)              #min eigenvalues
 lambda = min(0, lambda)
 print(lambda)
 
 
 SigmaET  = diag(SDs)%*% SigmaEcorr %*% diag(SDs)    #Thresholding
 SigmaETR = diag(SDs)%*% (SigmaEcorr - diag(lambda, 100)) %*% diag(SDs)/(1-lambda) 
                                                     #Thresholding and Regularization
 
 
 SigmaF = var(X)
 SigmaFactor   = B %*% SigmaF %*% t(B) + diag(diag(SigmaE))
 SigmaFactorT  = B %*% SigmaF %*% t(B) + SigmaET
 SigmaFactorTR = B %*% SigmaF %*% t(B) + SigmaETR
 
 EigenFactor   = eigen(SigmaFactor)$values
 EigenFactorT  = eigen(SigmaFactorT)$values
 EigenFactorTR = eigen(SigmaFactorTR)$values
 EigenSample = eigen(var(Y))$values
 
 

 postscript("Fig77.eps", width=4.4, height=4.8, horizontal=F, pointsize=8)
 par(mfrow = c(2,2), mar=c(4,2,2,1)+0.1, cex=0.8)
 
 hist(EigenSample[-1],nclass=30,    col=4, main="(a) Sample Covariance", 
   xlab = "Eigenvalues")
 hist(EigenFactor[-1], nclass=30,   col=4, main="(b) Factor Model",
   xlab="Eigenvalues")
 hist(EigenFactorT[-1], nclass=30,  col=4, main="(c) Approx Factor Model",
   xlab="Eigenvalues")
 hist(EigenFactorTR[-1], nclass=30, col=4, main="(d) Projection of Factor Model",
   xlab="Eigenvalues")

 max(EigenSample)
 max(EigenSample)/sum(EigenSample)
 
 sum(EigenSample < 0.0001)
 sum(EigenFactor < 0.0001)	
 sum(EigenFactorT < 0.0001)	
 sum(EigenFactorTR < 0.0001)	

